@@ -1,5 +1,6 @@
+/* SPDX-License-Identifier: GPL-2.0 */
 /*
- * Copyright (C) 2005-2017 Junjiro R. Okajima
+ * Copyright (C) 2005-2020 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +27,6 @@
 
 #include <linux/fsnotify.h>
 #include "rwsem.h"
-#include "vfsub.h"
 
 struct vfsmount;
 
@@ -37,6 +37,7 @@ struct au_hnotify {
 	struct fsnotify_mark		hn_mark;
 #endif
 	struct inode		*hn_aufs_inode;	/* no get/put */
+	struct rcu_head		rcu;
 #endif
 } ____cacheline_aligned_in_smp;
 
@@ -77,9 +78,10 @@ struct au_iinfo {
 };
 
 struct au_icntnr {
-	struct au_iinfo iinfo;
-	struct inode vfs_inode;
-	struct hlist_node plink;
+	struct au_iinfo		iinfo;
+	struct inode		vfs_inode;
+	struct hlist_bl_node	plink;
+	struct rcu_head		rcu;
 } ____cacheline_aligned_in_smp;
 
 /* au_pin flags */
@@ -155,7 +157,7 @@ enum {
 	AuIop_OTHER,
 	AuIop_Last
 };
-extern struct inode_operations aufs_iop[AuIop_Last],
+extern struct inode_operations aufs_iop[AuIop_Last], /* not const */
 	aufs_iop_nogetattr[AuIop_Last];
 
 /* au_wr_dir flags */
@@ -395,10 +397,9 @@ AuRWLockFuncs(new_child, NEW_CHILD);
 #undef AuWriteLockFunc
 #undef AuRWLockFuncs
 
-/*
- * ii_read_unlock, ii_write_unlock, ii_downgrade_lock
- */
-AuSimpleUnlockRwsemFuncs(ii, struct inode *i, &au_ii(i)->ii_rwsem);
+#define ii_read_unlock(i)	au_rw_read_unlock(&au_ii(i)->ii_rwsem)
+#define ii_write_unlock(i)	au_rw_write_unlock(&au_ii(i)->ii_rwsem)
+#define ii_downgrade_lock(i)	au_rw_dgrade_lock(&au_ii(i)->ii_rwsem)
 
 #define IiMustNoWaiters(i)	AuRwMustNoWaiters(&au_ii(i)->ii_rwsem)
 #define IiMustAnyLock(i)	AuRwMustAnyLock(&au_ii(i)->ii_rwsem)
@@ -612,7 +613,7 @@ void au_hn_free(struct au_hinode *hinode);
 void au_hn_ctl(struct au_hinode *hinode, int do_set);
 void au_hn_reset(struct inode *inode, unsigned int flags);
 int au_hnotify(struct inode *h_dir, struct au_hnotify *hnotify, u32 mask,
-	       struct qstr *h_child_qstr, struct inode *h_child_inode);
+	       const struct qstr *h_child_qstr, struct inode *h_child_inode);
 int au_hnotify_reset_br(unsigned int udba, struct au_branch *br, int perm);
 int au_hnotify_init_br(struct au_branch *br, int perm);
 void au_hnotify_fin_br(struct au_branch *br);
@@ -677,12 +678,15 @@ static inline void au_hn_inode_lock_nested(struct au_hinode *hdir,
 	au_hn_suspend(hdir);
 }
 
+#if 0 /* unused */
+#include "vfsub.h"
 static inline void au_hn_inode_lock_shared_nested(struct au_hinode *hdir,
 						  unsigned int sc)
 {
-	vfsub_inode_lock_shared_nested(hdir->hi_inode, sc);
+	inode_lock_shared_nested(hdir->hi_inode, sc);
 	au_hn_suspend(hdir);
 }
+#endif
 
 static inline void au_hn_inode_unlock(struct au_hinode *hdir)
 {
