@@ -1,5 +1,6 @@
+/* SPDX-License-Identifier: GPL-2.0 */
 /*
- * Copyright (C) 2005-2017 Junjiro R. Okajima
+ * Copyright (C) 2005-2020 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,8 +27,9 @@
 
 #include <linux/fs.h>
 #include <linux/kobject.h>
+#include "hbl.h"
+#include "lcnt.h"
 #include "rwsem.h"
-#include "spl.h"
 #include "wkq.h"
 
 /* policies to select one among multiple writable branches */
@@ -85,7 +87,7 @@ struct au_sbinfo {
 	struct au_nowait_tasks	si_nowait;
 
 	/*
-	 * tried sb->s_umount, but failed due to the dependecy between i_mutex.
+	 * tried sb->s_umount, but failed due to the dependency between i_mutex.
 	 * rwsem for au_sbinfo is necessary.
 	 */
 	struct au_rwsem		si_rwsem;
@@ -94,7 +96,7 @@ struct au_sbinfo {
 	 * dirty approach to protect sb->sb_inodes and ->s_files (gone) from
 	 * remount.
 	 */
-	struct percpu_counter	si_ninodes, si_nfiles;
+	au_lcnt_t		si_ninodes, si_nfiles;
 
 	/* branch management */
 	unsigned int		si_generation;
@@ -131,12 +133,14 @@ struct au_sbinfo {
 	/* external inode number (bitmap and translation table) */
 	vfs_readf_t		si_xread;
 	vfs_writef_t		si_xwrite;
+	loff_t			si_ximaxent;	/* max entries in a xino */
+
 	struct file		*si_xib;
 	struct mutex		si_xib_mtx; /* protect xib members */
 	unsigned long		*si_xib_buf;
 	unsigned long		si_xib_last_pindex;
 	int			si_xib_next_bit;
-	aufs_bindex_t		si_xino_brid;
+
 	unsigned long		si_xino_jiffy;
 	unsigned long		si_xino_expire;
 	/* reserved for future use */
@@ -144,12 +148,13 @@ struct au_sbinfo {
 
 #ifdef CONFIG_AUFS_EXPORT
 	/* i_generation */
+	/* todo: make xigen file an array to support many inode numbers */
 	struct file		*si_xigen;
 	atomic_t		si_xigen_next;
 #endif
 
-	/* dirty trick to suppoer atomic_open */
-	struct au_sphlhead	si_aopen;
+	/* dirty trick to support atomic_open */
+	struct hlist_bl_head	si_aopen;
 
 	/* vdir parameters */
 	unsigned long		si_rdcache;	/* max cache time in jiffies */
@@ -165,21 +170,21 @@ struct au_sbinfo {
 	unsigned int		si_dirwh;
 
 	/* pseudo_link list */
-	struct au_sphlhead	si_plink[AuPlink_NHASH];
+	struct hlist_bl_head	si_plink[AuPlink_NHASH];
 	wait_queue_head_t	si_plink_wq;
 	spinlock_t		si_plink_maint_lock;
 	pid_t			si_plink_maint_pid;
 
 	/* file list */
-	struct au_sphlhead	si_files;
+	struct hlist_bl_head	si_files;
 
 	/* with/without getattr, brother of sb->s_d_op */
-	struct inode_operations *si_iop_array;
+	const struct inode_operations *si_iop_array;
 
 	/*
 	 * sysfs and lifetime management.
 	 * this is not a small structure and it may be a waste of memory in case
-	 * of sysfs is disabled, particulary when many aufs-es are mounted.
+	 * of sysfs is disabled, particularly when many aufs-es are mounted.
 	 * but using sysfs is majority.
 	 */
 	struct kobject		si_kobj;
@@ -193,7 +198,7 @@ struct au_sbinfo {
 #endif
 
 #ifdef CONFIG_AUFS_SBILIST
-	struct hlist_node	si_list;
+	struct hlist_bl_node	si_list;
 #endif
 
 	/* dirty, necessary for unmounting, sysfs and sysrq */
@@ -204,7 +209,7 @@ struct au_sbinfo {
 /*
  * set true when refresh_dirs() failed at remount time.
  * then try refreshing dirs at access time again.
- * if it is false, refreshing dirs at access time is unnecesary
+ * if it is false, refreshing dirs at access time is unnecessary
  */
 #define AuSi_FAILED_REFRESH_DIR	1
 #define AuSi_FHSM		(1 << 1)	/* fhsm is active now */
@@ -345,7 +350,7 @@ int au_test_nfsd(void);
 void au_export_init(struct super_block *sb);
 void au_xigen_inc(struct inode *inode);
 int au_xigen_new(struct inode *inode);
-int au_xigen_set(struct super_block *sb, struct file *base);
+int au_xigen_set(struct super_block *sb, struct path *path);
 void au_xigen_clr(struct super_block *sb);
 
 static inline int au_busy_or_stale(void)
@@ -359,7 +364,7 @@ AuStubInt0(au_test_nfsd, void)
 AuStubVoid(au_export_init, struct super_block *sb)
 AuStubVoid(au_xigen_inc, struct inode *inode)
 AuStubInt0(au_xigen_new, struct inode *inode)
-AuStubInt0(au_xigen_set, struct super_block *sb, struct file *base)
+AuStubInt0(au_xigen_set, struct super_block *sb, struct path *path)
 AuStubVoid(au_xigen_clr, struct super_block *sb)
 AuStub(int, au_busy_or_stale, return -EBUSY, void)
 #endif /* CONFIG_AUFS_EXPORT */
@@ -368,32 +373,32 @@ AuStub(int, au_busy_or_stale, return -EBUSY, void)
 
 #ifdef CONFIG_AUFS_SBILIST
 /* module.c */
-extern struct au_sphlhead au_sbilist;
+extern struct hlist_bl_head au_sbilist;
 
 static inline void au_sbilist_init(void)
 {
-	au_sphl_init(&au_sbilist);
+	INIT_HLIST_BL_HEAD(&au_sbilist);
 }
 
 static inline void au_sbilist_add(struct super_block *sb)
 {
-	au_sphl_add(&au_sbi(sb)->si_list, &au_sbilist);
+	au_hbl_add(&au_sbi(sb)->si_list, &au_sbilist);
 }
 
 static inline void au_sbilist_del(struct super_block *sb)
 {
-	au_sphl_del(&au_sbi(sb)->si_list, &au_sbilist);
+	au_hbl_del(&au_sbi(sb)->si_list, &au_sbilist);
 }
 
 #ifdef CONFIG_AUFS_MAGIC_SYSRQ
 static inline void au_sbilist_lock(void)
 {
-	spin_lock(&au_sbilist.spin);
+	hlist_bl_lock(&au_sbilist);
 }
 
 static inline void au_sbilist_unlock(void)
 {
-	spin_unlock(&au_sbilist.spin);
+	hlist_bl_unlock(&au_sbilist);
 }
 #define AuGFP_SBILIST	GFP_ATOMIC
 #else
@@ -433,7 +438,7 @@ static inline void dbgaufs_si_null(struct au_sbinfo *sbinfo)
 
 /* current->atomic_flags */
 /* this value should never corrupt the ones defined in linux/sched.h */
-#define PFA_AUFS	7
+#define PFA_AUFS	0x10
 
 TASK_PFA_TEST(AUFS, test_aufs)	/* task_test_aufs */
 TASK_PFA_SET(AUFS, aufs)	/* task_set_aufs */
@@ -459,11 +464,20 @@ static inline void si_pid_set(struct super_block *sb)
 /* ---------------------------------------------------------------------- */
 
 /* lock superblock. mainly for entry point functions */
+#define __si_read_lock(sb)	au_rw_read_lock(&au_sbi(sb)->si_rwsem)
+#define __si_write_lock(sb)	au_rw_write_lock(&au_sbi(sb)->si_rwsem)
+#define __si_read_trylock(sb)	au_rw_read_trylock(&au_sbi(sb)->si_rwsem)
+#define __si_write_trylock(sb)	au_rw_write_trylock(&au_sbi(sb)->si_rwsem)
 /*
- * __si_read_lock, __si_write_lock,
- * __si_read_unlock, __si_write_unlock, __si_downgrade_lock
- */
-AuSimpleRwsemFuncs(__si, struct super_block *sb, &au_sbi(sb)->si_rwsem);
+#define __si_read_trylock_nested(sb) \
+	au_rw_read_trylock_nested(&au_sbi(sb)->si_rwsem)
+#define __si_write_trylock_nested(sb) \
+	au_rw_write_trylock_nested(&au_sbi(sb)->si_rwsem)
+*/
+
+#define __si_read_unlock(sb)	au_rw_read_unlock(&au_sbi(sb)->si_rwsem)
+#define __si_write_unlock(sb)	au_rw_write_unlock(&au_sbi(sb)->si_rwsem)
+#define __si_downgrade_lock(sb)	au_rw_dgrade_lock(&au_sbi(sb)->si_rwsem)
 
 #define SiMustNoWaiters(sb)	AuRwMustNoWaiters(&au_sbi(sb)->si_rwsem)
 #define SiMustAnyLock(sb)	AuRwMustAnyLock(&au_sbi(sb)->si_rwsem)
@@ -558,42 +572,6 @@ static inline unsigned int au_sigen(struct super_block *sb)
 	return au_sbi(sb)->si_generation;
 }
 
-static inline unsigned long long au_ninodes(struct super_block *sb)
-{
-	s64 n = percpu_counter_sum(&au_sbi(sb)->si_ninodes);
-
-	BUG_ON(n < 0);
-	return n;
-}
-
-static inline void au_ninodes_inc(struct super_block *sb)
-{
-	percpu_counter_inc(&au_sbi(sb)->si_ninodes);
-}
-
-static inline void au_ninodes_dec(struct super_block *sb)
-{
-	percpu_counter_dec(&au_sbi(sb)->si_ninodes);
-}
-
-static inline unsigned long long au_nfiles(struct super_block *sb)
-{
-	s64 n = percpu_counter_sum(&au_sbi(sb)->si_nfiles);
-
-	BUG_ON(n < 0);
-	return n;
-}
-
-static inline void au_nfiles_inc(struct super_block *sb)
-{
-	percpu_counter_inc(&au_sbi(sb)->si_nfiles);
-}
-
-static inline void au_nfiles_dec(struct super_block *sb)
-{
-	percpu_counter_dec(&au_sbi(sb)->si_nfiles);
-}
-
 static inline struct au_branch *au_sbr(struct super_block *sb,
 				       aufs_bindex_t bindex)
 {
@@ -601,16 +579,10 @@ static inline struct au_branch *au_sbr(struct super_block *sb,
 	return au_sbi(sb)->si_branch[0 + bindex];
 }
 
-static inline void au_xino_brid_set(struct super_block *sb, aufs_bindex_t brid)
-{
-	SiMustWriteLock(sb);
-	au_sbi(sb)->si_xino_brid = brid;
-}
-
-static inline aufs_bindex_t au_xino_brid(struct super_block *sb)
+static inline loff_t au_xi_maxent(struct super_block *sb)
 {
 	SiMustAnyLock(sb);
-	return au_sbi(sb)->si_xino_brid;
+	return au_sbi(sb)->si_ximaxent;
 }
 
 #endif /* __KERNEL__ */
